/***************************************************************************//**
 *   @file   Main.c
 *   @brief  Implementation of the program's main function.
 *   @author Dan Nechita
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
 *   SVN Revision: 781
*******************************************************************************/

/******************************************************************************/
/***************************** Option Bytes ***********************************/
/******************************************************************************/
#pragma location = "OPTBYTE"
__root const unsigned char opbyte0 = 0x00; // Watchdog timer settings.
#pragma location = "OPTBYTE"
__root const unsigned char opbyte1 = 0xFD; // Low voltage detector settings.
#pragma location = "OPTBYTE"
__root const unsigned char opbyte2 = 0xF8; // Flash op. mode and osc. settings.
#pragma location = "OPTBYTE"
__root const unsigned char opbyte3 = 0x04; // On-chip Debug Option Byte.

/******************************************************************************/
/***************************** Include Files **********************************/
/******************************************************************************/
#include "YRDKRL78G13.h"    // YRDKRL78G13 definitions.
#include "ST7579.h"         // ST7579 definitions.
#include "TIME.h"           // TIME definitions.
#include "AD57XX.h"         // AD57XX definitions.

/***************************************************************************//**
* @brief Draws the ADI logo and displays the name of ADI component.
*
* @param componentName - The name of the ADI component.
*
* @return None.
*******************************************************************************/
void ADI_Component(unsigned char* string)
{
    const unsigned char adiComponent[7] =
    {
        0xFE, 0xFE, 0x82, 0xC6, 0xEE, 0xFE, 0xFE 
    };
    ST7579_Image(0, 0, adiComponent, 7, 8);
    ST7579_String(0, 9, string, 0);
}

/***************************************************************************//**
 * @brief Main function.
 *
 * @return None.
*******************************************************************************/
void main(void)
{    
    long dacVal  = 0;
    long step    = 0;
    long maxCode = 0x3FFFF;    // 18 bits
    long minCode = 0;
    long result  = 0;
    
    /* Initialize YRDKRL78G13. */
    YRDKRL78G13_Init();
    
    /* Enable interrupts. */
    __enable_interrupt();
        
    /* Initialize timer. */
    TIME_Init();
    
    /* Initialize the ST7579 Display. */
    ST7579_Init();
    
    /* Initialize AD57XX device. */
    if(AD57XX_Init(AD5780))
    {
        ADI_Component("AD57XX OK");
    }
    else
    {
        ADI_Component("AD57XX Err");
    }
    
    /* Resets the device to its power-on state. */
    AD57XX_SoftInstruction(AD57XX_SOFT_CTRL_RESET);
    
    /* Enables the DAC output. */
    AD57XX_EnableOutput(1);
    
    /* The DAC register is set to use offset binary coding. */
    AD57XX_Setup(AD57XX_CTRL_BIN2SC);
    
    /* Sets the value to which the DAC output is set when CLEAR is enabled. */
    AD57XX_SetClearCode(0x20000);  
    
    /* Performs a soft CLEAR operation. */
     AD57XX_SoftInstruction(AD57XX_SOFT_CTRL_CLR);
     
    /* Reads and displays the internal registers. */
     /* Read DAC. */
    result = AD57XX_GetRegisterValue(AD57XX_REG_DAC);
    result = (result & 0xFFFFC) >> 2;
    ST7579_String(2, 0, "DAC REG:", 0);
    ST7579_HexNumber(3, 0, result, 0);
    
    /* Read Control. */
    result = AD57XX_GetRegisterValue(AD57XX_REG_CTRL);
    result = (result & 0x3E);  // Only Bits 5 through 1 are holding information.
    ST7579_String(4, 0, "CTRL REG:", 0);
    ST7579_HexNumber(5, 0, result, 0);
    
    /* Read ClearCode. */
    result = AD57XX_GetRegisterValue(AD57XX_REG_CLR_CODE);
    result = (result & 0xFFFFC) >> 2;
    ST7579_String(6, 0, "CLEAR CODE:", 0);
    ST7579_HexNumber(7, 0, result, 0);
    
    /* Generates a triangle signal. */
    while(1)
    {
        if(dacVal >= maxCode - step)
       {
           step = -250;
       }
       else if(dacVal <= minCode - step)
            {
                step = 250;
            }
       dacVal += step;       
       AD57XX_SetDacValue(dacVal);
       TIME_DelayUs(50);
    }
}
